/**
 * A basic 2D Vector class that provides simple algebraic functionality in the form
 * of 2D Vectors.
 *
 * We use Getters/setters for both principle properties (x & y) as well as virtual
 * properties (rotation, length etc.).
 *
 * @class Vec2
 * @author Liam Egan <liam@wethecollective.com>
 * @version 1.0.0
 * @created Jan 07, 2020
 */
declare class Vec2 {
    #private;
    /**
     * The Vector Class constructor
     *
     * @constructor
     * @chainable
     * @param {number} x 				The x coord
     * @param {number} y 				The y coord
     */
    constructor(...args: number[]);
    /**
     * Resets the vector coordinates
     *
     * @public
     * @chainable
     * @param {number|Array} x 	The x coord, OR the array to reset to
     * @param {number} y 				The y coord
     */
    reset(...args: number[]): Vec2;
    /**
     * Resets the vector coordinates to another vector object
     *
     * @public
     * @chainable
     * @param {Vector} v 				The vector object to use to reset the coordinates
     */
    resetToVector(v: Vec2): Vec2;
    /**
     * Clones the vector
     *
     * @public
     * @chainable
     * @return {Vector}					The cloned vector
     */
    clone(): Vec2;
    /**
     * Adds one vector to another.
     *
     * @public
     * @chainable
     * @param  {Vec2}  vector The vector to add to this one
     * @return {Vec2}					Returns itself, modified
     */
    add(vector: Vec2): Vec2;
    /**
     * Clones the vector and adds the vector to it instead
     *
     * @public
     * @chainable
     * @param  {Vec2}  vector The vector to add to this one
     * @return {Vec2}					Returns the clone of itself, modified
     */
    addNew(vector: Vec2): Vec2;
    /**
     * Adds a scalar to the vector, modifying both the x and y
     *
     * @public
     * @chainable
     * @param  {number}  scalar The scalar to add to the vector
     * @return {Vec2}					Returns itself, modified
     */
    addScalar(scalar: number): Vec2;
    /**
     * Clones the vector and adds the scalar to it instead
     *
     * @public
     * @chainable
     * @param  {number}  scalar The scalar to add to the vector
     * @return {Vec2}					Returns the clone of itself, modified
     */
    addScalarNew(scalar: number): Vec2;
    /**
     * Subtracts one vector from another.
     *
     * @public
     * @chainable
     * @param  {Vec2}  vector The vector to subtract from this one
     * @return {Vec2}					Returns itself, modified
     */
    subtract(vector: Vec2): Vec2;
    /**
     * Clones the vector and subtracts the vector from it instead
     *
     * @public
     * @chainable
     * @param  {Vec2}  vector The vector to subtract from this one
     * @return {Vec2}					Returns the clone of itself, modified
     */
    subtractNew(vector: Vec2): Vec2;
    /**
     * Subtracts a scalar from the vector, modifying both the x and y
     *
     * @public
     * @chainable
     * @param  {number}  scalar The scalar to subtract from the vector
     * @return {Vec2}					Returns itself, modified
     */
    subtractScalar(scalar: number): Vec2;
    /**
     * Clones the vector and subtracts the scalar from it instead
     *
     * @public
     * @chainable
     * @param  {number}  scalar The scalar to add to the vector
     * @return {Vec2}					Returns the clone of itself, modified
     */
    subtractScalarNew(scalar: number): Vec2;
    /**
     * Divides one vector by another.
     *
     * @public
     * @chainable
     * @param  {Vec2}  vector The vector to divide this by
     * @return {Vec2}					Returns itself, modified
     */
    divide(vector: Vec2): Vec2;
    /**
     * Clones the vector and divides it by the vector instead
     *
     * @public
     * @chainable
     * @param  {Vec2}  vector The vector to divide the clone by
     * @return {Vec2}					Returns the clone of itself, modified
     */
    divideNew(vector: Vec2): Vec2;
    /**
     * Divides the vector by a scalar.
     *
     * @public
     * @chainable
     * @param  {number}  scalar The scalar to divide both x and y by
     * @return {Vec2}					Returns itself, modified
     */
    divideScalar(scalar: number): Vec2;
    /**
     * Clones the vector and divides it by the provided scalar.
     *
     * @public
     * @chainable
     * @param  {number}  scalar The scalar to divide both x and y by
     * @return {Vec2}					Returns the clone of itself, modified
     */
    divideScalarNew(scalar: number): Vec2;
    /**
     * Multiplies one vector by another.
     *
     * @public
     * @chainable
     * @param  {Vec2}  vector The vector to multiply this by
     * @return {Vec2}					Returns itself, modified
     */
    multiply(vector: Vec2): Vec2;
    /**
     * Clones the vector and multiplies it by the vector instead
     *
     * @public
     * @chainable
     * @param  {Vec2}  vector The vector to multiply the clone by
     * @return {Vec2}					Returns the clone of itself, modified
     */
    multiplyNew(vector: Vec2): Vec2;
    /**
     * Multiplies the vector by a scalar.
     *
     * @public
     * @chainable
     * @param  {number}  scalar The scalar to multiply both x and y by
     * @return {Vec2}					Returns itself, modified
     */
    multiplyScalar(scalar: number): Vec2;
    /**
     * Clones the vector and multiplies it by the provided scalar.
     *
     * @public
     * @chainable
     * @param  {number}  scalar The scalar to multiply both x and y by
     * @return {Vec2}					Returns the clone of itself, modified
     */
    multiplyScalarNew(scalar: number): Vec2;
    /**
     * Alias of {@link Vector#multiplyScalar__anchor multiplyScalar}
     */
    scale(scalar: number): Vec2;
    /**
     * Alias of {@link Vector#multiplyScalarNew__anchor multiplyScalarNew}
     */
    scaleNew(scalar: number): Vec2;
    /**
     * Rotates a vecor by a given amount, provided in radians.
     *
     * @public
     * @chainable
     * @param  {number}  radian The angle, in radians, to rotate the vector by
     * @return {Vec2}					Returns itself, modified
     */
    rotate(radian: number): Vec2;
    /**
     * Clones the vector and rotates it by the supplied radian value
     *
     * @public
     * @chainable
     * @param  {number}  radian The angle, in radians, to rotate the vector by
     * @return {Vec2}					Returns the clone of itself, modified
     */
    rotateNew(radian: number): Vec2;
    /**
     * Rotates a vecor by a given amount, provided in degrees. Converts the degree
     * value to radians and runs the rotaet method.
     *
     * @public
     * @chainable
     * @param  {number}  degrees The angle, in degrees, to rotate the vector by
     * @return {Vec2}						Returns itself, modified
     */
    rotateDeg(degrees: number): Vec2;
    /**
     * Clones the vector and rotates it by the supplied degree value
     *
     * @public
     * @chainable
     * @param  {number}  degrees The angle, in degrees, to rotate the vector by
     * @return {Vec2}					 Returns the clone of itself, modified
     */
    rotateDegNew(degrees: number): Vec2;
    /**
     * Alias of {@link Vector#rotate__anchor rotate}
     */
    rotateBy(radian: number): Vec2;
    /**
     * Alias of {@link Vector#rotateNew__anchor rotateNew}
     */
    rotateByNew(radian: number): Vec2;
    /**
     * Alias of {@link Vector#rotateDeg__anchor rotateDeg}
     */
    rotateDegBy(degrees: number): Vec2;
    /**
     * Alias of {@link Vector#rotateDegNew__anchor rotateDegNew}
     */
    rotateDegByNew(radian: number): Vec2;
    /**
     * Rotates a vector to a specific angle
     *
     * @public
     * @chainable
     * @param  {number}  radian The angle, in radians, to rotate the vector to
     * @return {Vec2}					Returns itself, modified
     */
    rotateTo(radian: number): Vec2;
    /**
     * Clones the vector and rotates it to the supplied radian value
     *
     * @public
     * @chainable
     * @param  {number}  radian The angle, in radians, to rotate the vector to
     * @return {Vec2}					Returns the clone of itself, modified
     */
    rotateToNew(radian: number): Vec2;
    /**
     * Rotates a vecor to a given amount, provided in degrees. Converts the degree
     * value to radians and runs the rotateTo method.
     *
     * @public
     * @chainable
     * @param  {number}  degrees The angle, in degrees, to rotate the vector to
     * @return {Vec2}						Returns itself, modified
     */
    rotateToDeg(degrees: number): Vec2;
    /**
     * Clones the vector and rotates it to the supplied degree value
     *
     * @public
     * @chainable
     * @param  {number}  degrees The angle, in degrees, to rotate the vector to
     * @return {Vec2}					 Returns the clone of itself, modified
     */
    rotateToDegNew(degrees: number): Vec2;
    /**
     * Negates the vector.
     *
     * @public
     * @chainable
     * @return {Vec2}					Returns itself, modified
     */
    negate(): Vec2;
    /**
     * Clones the vector and negates it.
     *
     * @public
     * @chainable
     * @return {Vec2}					Returns itself, modified
     */
    negateNew(): Vec2;
    /**
     * Inverses the vector.
     *
     * @public
     * @chainable
     * @return {Vec2}					Returns itself, modified
     */
    inverse(): Vec2;
    /**
     * Clones the vector and then inverses it.
     *
     * @public
     * @chainable
     * @return {Vec2}					Returns itself, modified
     */
    inverseNew(): Vec2;
    /**
     * Normalises the vector down to a length of 1 unit
     *
     * @public
     * @chainable
     * @return {Vec2}					Returns itself, modified
     */
    normalise(): Vec2;
    /**
     * Clones the vector and normalises it
     *
     * @public
     * @chainable
     * @return {Vec2}					Returns a clone of itself, modified
     */
    normaliseNew(): Vec2;
    /**
     * Calculates the distance between this and the supplied vector
     *
     * @param  {Vec2} vector The vector to calculate the distance from
     * @return {number}        The distance between this and the supplied vector
     */
    distance(vector: Vec2): number;
    /**
     * Calculates the distance on the X axis between this and the supplied vector
     *
     * @param  {Vec2} vector The vector to calculate the distance from
     * @return {number}        The distance, along the x axis, between this and the supplied vector
     */
    distanceX(vector: Vec2): number;
    /**
     * Calculated the distance on the Y axis between this and the supplied vector
     *
     * @param  {Vec2} vector The vector to calculate the distance from
     * @return {number}        The distance, along the y axis, between this and the supplied vector
     */
    distanceY(vector: Vec2): number;
    /**
     * Calculates the dot product between this and a supplied vectorT
     *
     * @example
     * // returns -14
     * new Vector(2, -3).dot(new Vector(-4, 2))
     * new Vector(-4, 2).dot(new Vector(2, -3))
     * new Vector(2, -4).dot(new Vector(-3, 2))
     *
     * @param  {Vec2} vector The vector object against which to calculate the dot product
     * @return {number}        The dot product of the two vectors
     */
    dot(vector: Vec2): number;
    det(vector: Vec2): number;
    slopeBetween(vector: Vec2): number;
    /**
     * Calculates the cross product between this and the supplied vector.
     *
     * @example
     * // returns -2
     * new Vector(2, -3).cross(new Vector(-4, 2))
     * new Vector(-4, 2).cross(new Vector(2, -3))
     * // returns 2
     * new Vector(2, -4).cross(new Vector(-3, 2))
     *
     * @param  {Vec2} vector The vector object against which to calculate the cross product
     * @return {number}        The cross product of the two vectors
     */
    cross(vector: Vec2): number;
    ceil(): Vec2;
    ceilNew(): Vec2;
    floor(): Vec2;
    floorNew(): Vec2;
    round(): Vec2;
    roundNew(): Vec2;
    mod(vector: Vec2): Vec2;
    modNew(vector: Vec2): Vec2;
    fract(): Vec2;
    fractNew(): Vec2;
    transformByMat2(m: any): Vec2;
    transformByMat2New(m: any): Vec2;
    transformByMat3(m: any): Vec2;
    transformByMat3New(m: any): Vec2;
    set x(x: number);
    get x(): number;
    set y(y: number);
    get y(): number;
    /**
     * (getter/setter) The length of the vector presented as a square. If you're using
     * length for comparison, this is quicker.
     *
     * @type {number}
     * @default 0
     */
    set lengthSquared(length: number);
    get lengthSquared(): number;
    /**
     * (getter/setter) The length of the vector
     *
     * @type {number}
     * @default 0
     */
    set length(length: number);
    get length(): number;
    /**
     * (getter/setter) The angle of the vector, in radians
     *
     * @type {number}
     * @default 0
     */
    set angle(radian: number);
    get angle(): number;
    /**
     * (getter/setter) The angle of the vector, in radians
     *
     * @type {number}
     * @default 0
     */
    set angleInDegrees(degrees: number);
    get angleInDegrees(): number;
    /**
     * (getter/setter) Vector width.
     * Alias of {@link Vector#x x}
     *
     * @type {number}
     */
    set width(w: number);
    get width(): number;
    /**
     * (getter/setter) Vector height.
     * Alias of {@link Vector#x x}
     *
     * @type {number}
     */
    set height(h: number);
    get height(): number;
    /**
     * (getter) Vector area.
     * @readonly
     *
     * @type {number}
     */
    get area(): number;
    /**
     * (getter/setter) Vector slope.
     *
     * @type {number}
     */
    set slope(value: number);
    get slope(): number;
    /**
     * (getter) Returns the basic array representation of this vector.
     * @readonly
     *
     * @type {number}
     */
    get array(): number[];
    /**
     * (getter/sette) Swizzle XY
     *
     * @type {number}
     */
    get xy(): Vec2 | number[];
    set xy(v: Vec2 | number[]);
    /**
     * (getter/sette) Swizzle YX
     *
     * @type {number}
     */
    get yx(): any;
    set yx(v: any);
    /**
     * (getter/sette) Swizzle XX
     *
     * @type {number}
     */
    get xx(): any;
    set xx(v: any);
    /**
     * (getter/sette) Swizzle YY
     *
     * @type {number}
     */
    get yy(): any;
    set yy(v: any);
    /**
     * Static methods
     */
    /**
     * Iterpolates a provided anonymous value into a vew Vec2
     *
     * @param {Vec2|array|string|number} v The value to interpolate
     * @returns {Vec2} out
     */
    static interpolate(v: any): Vec2;
    /**
     * Performs a linear interpolation between two vec2's
     *
     * @param {vec2} v1 the first operand
     * @param {vec2} v2 the second operand
     * @param {Number} d interpolation amount in the range of 0 - 1
     * @returns {Vec2}
     */
    static lerp(v1: Vec2, v2: Vec2, d: number): Vec2;
    /**
     * Finds the angle between 2 vectors.
     *
     * @param {vec2} a the first operand
     * @param {vec2} b the second operand
     * @returns {number}
     */
    static getAngle(a: Vec2, b: Vec2): number;
}
export { Vec2 };
